%---------------------------------------------------%
% Foot Motion Problem:                              %
%---------------------------------------------------%

clear all; close all;

calibrate = true; % If calibrate is not true, then the problem is verifying that it gets back 
extraControls = false;
handLoadControls = true; % You can't have extra controls and hand load controls at the same time
springPosParams = false;
% addpath('C:\Files\GPOPS-II New\nlp\snopt')

% Load model parameters determined by past calibration runs
cdorig = cd;
cd('../SetupFiles')
if calibrate
    auxdata = SetupOpenSimOptimalTracking(0,false);
else
    auxdata = SetupOpenSimOptimalTracking(0,true);
end
cd(cdorig)

auxdata.calibrate = calibrate;
auxdata.extraControls = extraControls;
auxdata.handLoadControls = handLoadControls;
auxdata.springPosParams = springPosParams;

% Load initial and final model pose and time
t0 = auxdata.time(1);
tf = auxdata.time(end);
time = auxdata.time;
q0 = auxdata.q(1,:);
qp0 = auxdata.qp(1,:);
qf = auxdata.q(end,:);
qpf = auxdata.qp(end,:);

% Set initial guesses for Loads (control) and joint angle trajectories
% (states)
loadsGuess = auxdata.idData;
qGuess = auxdata.q;
qpGuess = auxdata.qp;
qppGuess = auxdata.qpp;
qpppGuess = auxdata.qppp;
nJerkControls = size(qpppGuess,2);
auxdata.nJerkControls = nJerkControls;

% Determine the range of the states
qRange = max(qGuess) - min(qGuess);
qpRange = max(qpGuess) - min(qpGuess);
qppRange = max(qppGuess) - min(qppGuess);
qpppRange = max(qpppGuess) - min(qpppGuess);

auxdata.activeDofs = logical(1-(qRange == 0));
auxdata.activeDofs(1:6) = [];

% Give locked DOFs a range of motion
qpppRange(qRange == 0) = 40000*pi/180;
qppRange(qRange == 0) = 4000*pi/180;
qpRange(qRange == 0) = 400*pi/180;
qRange(qRange == 0) = 60*pi/180;

% Set bounds on the states based on the previously calcualted range
qMin = min(qGuess) - 2*qRange;  
qMax  = max(qGuess) + 2*qRange;
qpMin = min(qpGuess) - 1.5*qpRange; 
qpMax  = max(qpGuess) + 1.5*qpRange;
qppMin = min(qppGuess) - 1*qppRange; 
qppMax  = max(qppGuess) + 1*qppRange;
qpppMin = min(qpppGuess) - 1*qpppRange; 
qpppMax  = max(qpppGuess) + 1*qpppRange;

% Determing the range of the controls
loadsRange = max(auxdata.idData) - min(auxdata.idData);
torquesRange = max(auxdata.torques) - min(auxdata.torques);

% Give bigger range to DOFs with small loads (for instance the toes)
loadsRange(loadsRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);
torquesRange(torquesRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);

% Set bounds on the torques (for when using extra controls)
loadsMin = min(auxdata.idData)-2*loadsRange; 
loadsMax = max(auxdata.idData)+2*loadsRange;

torquesMin = min(auxdata.torques) - 2*torquesRange;
torquesMax = max(auxdata.torques) + 2*torquesRange;

torquesGuess = auxdata.torques;

% Set weights for marker errors (the model tracks markers from experimental
% data)
% NOTE (Nathan): These are not actually being used anywhere!
auxdata.errorWeights = ones(1,numel(auxdata.mkrNames));
auxdata.errorWeights([18 21 28 31]) = 10;
auxdata.errorWeights([3:8]) = .1;

auxdata.errorWeights = repmat(auxdata.errorWeights,3,1);
auxdata.errorWeights = auxdata.errorWeights(:)';

% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt1_GPOPS.osim');
opensimID_OMP('Subject1_v4pt1_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt1_GPOPS.osim');

% Get median hand positions
handForceLoc = [0.01,-0.07,0; 0.01,-0.07,0]; % NOTE (Nathan): Make sure that this is the same location as where the force is applied in the model
auxdata.handForceLoc = handForceLoc;
handBodyMat = auxdata.handBodyMat;
% [handPos, handVel, handAng, handAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],handBodyMat);
[handPos, handVel, handAng, handAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],handForceLoc,handBodyMat);


medianHandPos = median(handPos);
medianHandAng = median(handAng);
auxdata.medianHandPos_r = medianHandPos(1:3);
auxdata.medianHandPos_l = medianHandPos(4:6);
auxdata.medianHandAng_r = medianHandAng(1:3);
auxdata.medianHandAng_l = medianHandAng(4:6);

% Min and Max hand loads allowed
% NOTE (Nathan): Thses are only used if the hand loads are minimized or for
% hand load controls
handLoadMin = -[200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadMax = [200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadRange = handLoadMax - handLoadMin;

% Marker Errors
if calibrate
    mkrErrorsAllowed = 2e-2*ones(1,numel(auxdata.mkrNames));
    mkrErrorsAllowed([18:21 28:31]) = 1.5e-2;
    mkrErrorsAllowed([3:8]) = 3e-2;
else
    mkrErrorsAllowed = 2e-2*ones(1,numel(auxdata.mkrNames));
    mkrErrorsAllowed([18:21 28:31]) = 1.5e-2;
    mkrErrorsAllowed([3:8]) = 3e-2;
end
mkrErrorsAllowed = repmat(mkrErrorsAllowed, 3,1);% Duplicate for each dimension
mkrErrorsAllowed = mkrErrorsAllowed(:)';

auxdata.numSprings = sum(auxdata.numSpringsBody);

% load previous solution for use as an initial guess
% load solution_osimOptTracking_calibrateSprings29.mat
auxdata.beltSpeed = 0.6;

% solution = output.result.interpsolution;

% Setup scale factors
auxdata.tMin = t0;
auxdata.tMax = tf;
auxdata.stateMin = [qMin,qpMin,qppMin];
auxdata.stateMax = [qMax,qpMax,qppMax];
if extraControls
    auxdata.controlMin = [qpppMin,torquesMin];
    auxdata.controlMax = [qpppMax,torquesMax];
elseif handLoadControls
    auxdata.controlMin = [qpppMin,handLoadMin];
    auxdata.controlMax = [qpppMax,handLoadMax];
else
    auxdata.controlMin = qpppMin;
    auxdata.controlMax = qpppMax;
end
if handLoadControls
    auxdata.integralMin = 0*ones(1,99+6+6+12+2+12+31);
    auxdata.integralMax = auxdata.tMax*([mkrErrorsAllowed ...
        1e0*[1e1 2e1 1e1]...
        1e0*[1e1 2e1 1e1]... 
        1e1*ones(1,3)...
        1e1*ones(1,3)...
        1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
        1*qRange([13,20]) ... % Toe angle tracking
        handLoadRange...
        1*qpppRange]).^2; % Ignore locked coordinates (wrists) and patella OLD: all
else
    auxdata.integralMin = 0*ones(1,99+6+6+12+2+31); %99+6+6+23+31
    auxdata.integralMax = auxdata.tMax*([mkrErrorsAllowed ...
        1e0*[1e1 2e1 1e1]...
        1e0*[1e1 2e1 1e1]... 
        1e1*ones(1,3)...
        1e1*ones(1,3)...
        1*loadsRange([7:10,12:13,15:18,20:21])... %  [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
        1*qRange([13,20]) ... % Toe angle tracking
        1*qpppRange]).^2; % Ignore locked coordinates (wrists) and patella OLD: all
end
   
onescol = ones(length(auxdata.time),1);
numTorques = size(auxdata.torques,2);
if extraControls
    auxdata.pathMin = 1e0*[-1e-1*ones(1,3),-1e-0*ones(1,3),-1e-5*ones(1,numTorques)];
    auxdata.pathMax = 1e0*[1e-1*ones(1,3),1e-0*ones(1,3),1e-5*ones(1,numTorques)];
else
    auxdata.pathMin = 1*[-1e-1*ones(1,3) -1e0*ones(1,3)]; % 0.11*
    auxdata.pathMax = 1*[1e-1*ones(1,3) 1e0*ones(1,3)];
end
if calibrate
    if springPosParams
%         params: [GRF kVals(x6), GRF yVals(x2), hand spring kVals(x3),
%         torsion hand spring kVals(x3), right hand pos correc, right
%         hand ang correct(x3), left hand pos correct(x3), left hand ang
%         correct(x3)]
        auxdata.paramMin = [0 -2e3*ones(1,5) -0.01*ones(1,2) ...
            1*ones(1,3) 1*ones(1,3) -0.01*ones(1,3) -0.5*ones(1,3) -0.01*ones(1,3) -0.5*ones(1,3)];
        auxdata.paramMax = [1e4 2e3*ones(1,5) 0.01*ones(1,2) ...
            1e4*ones(1,3) 1e3*ones(1,3) 0.01*ones(1,3) 0.5*ones(1,3) 0.01*ones(1,3) 0.5*ones(1,3)];
    elseif ~handLoadControls
        % Specify bounds of spring constants
%         params: [GRF kVals(x6), GRF yVals(x2), hand spring kVals(x3),
%         torsion hand spring kVals(x3)]
    %     auxdata.paramMin = [0 -2e3*ones(1,5) -0.01*ones(1,2) 1*ones(1,6)];
    %     auxdata.paramMax = [1e4 2e3*ones(1,5) 0.01*ones(1,2) 1e4*ones(1,6)];
        auxdata.paramMin = [0 -2e3*ones(1,5) -0.01*ones(1,2) 1*ones(1,3) 1*ones(1,3)];
        auxdata.paramMax = [1e4 2e3*ones(1,5) 0.01*ones(1,2) 1e4*ones(1,3) 1e3*ones(1,3)];
    %     auxdata.paramMin = [0 -2e3*ones(1,5) -0.01*ones(1,2)];
    %     auxdata.paramMax = [1e4 2e3*ones(1,5) 0.01*ones(1,2)];
    else
        auxdata.paramMin = [0 -4e3*ones(1,5) -0.01*ones(1,2)]; %[0 -2e3*ones(1,5) -0.01*ones(1,2)]
        auxdata.paramMax = [1e4 4e3*ones(1,5) 0.01*ones(1,2)]; %[1e4 2e3*ones(1,5) 0.01*ones(1,2)]
    end
    
end


% Load Previous Solution
load('solution_calibrateSprings_snopt_v10HandControlsPt15.mat');

if 0 %Run Opt    
%-------------------------------------------------------------------------%
%----------------------- Setup for Problem Bounds ------------------------%
%-------------------------------------------------------------------------%

nStates = length(auxdata.stateMin);

% nparams = length(auxdata.paramsmin);

iphase = 1;
bounds.phase(iphase).initialtime.lower = -.5;
bounds.phase(iphase).initialtime.upper = -.5;
bounds.phase(iphase).finaltime.lower = .5;
bounds.phase(iphase).finaltime.upper = .5;
% states are the joint angles, velocities, accelerations and the joint
% torques
bounds.phase(iphase).initialstate.lower = -.5*ones(1,nStates);%,vmin];
bounds.phase(iphase).initialstate.upper = .5*ones(1,nStates);%,vmax];
bounds.phase(iphase).finalstate.lower = -.5*ones(1,nStates);%,vmin];%[qf,qpf];
bounds.phase(iphase).finalstate.upper = .5*ones(1,nStates);%,vmax];%[qf,qpf];
bounds.phase(iphase).state.lower = -.5*ones(1,nStates);%,vmin];
bounds.phase(iphase).state.upper = .5*ones(1,nStates);%,vmax];
% Dynamics equations are forced to be satisfied by path constraints
% Control with joint angle jerk and the time derivative of the joint loads
if extraControls
    bounds.phase(iphase).control.lower = -.5*ones(1,nJerkControls+numTorques);
    bounds.phase(iphase).control.upper = .5*ones(1,nJerkControls+numTorques);
    bounds.phase(iphase).path.lower = -0.5*ones(1,6+numTorques); % Bounds on pelvis residuals
    bounds.phase(iphase).path.upper = 0.5*ones(1,6+numTorques);
elseif handLoadControls
    bounds.phase(iphase).control.lower = -.5*ones(1,nJerkControls+12);
    bounds.phase(iphase).control.upper = .5*ones(1,nJerkControls+12);
    bounds.phase(iphase).path.lower = -0.5*ones(1,6); % Bounds on pelvis residuals
    bounds.phase(iphase).path.upper = 0.5*ones(1,6);
else
    bounds.phase(iphase).control.lower = -.5*ones(1,nJerkControls);
    bounds.phase(iphase).control.upper = .5*ones(1,nJerkControls);
    bounds.phase(iphase).path.lower = -0.5*ones(1,6); % Bounds on pelvis residuals
    bounds.phase(iphase).path.upper = 0.5*ones(1,6);
end
bounds.phase(iphase).integral.lower = 0*(ones(size(auxdata.integralMin)));
bounds.phase(iphase).integral.upper = 1*(ones(size(auxdata.integralMax)));
if calibrate
    if springPosParams
        bounds.parameter.lower = [-2*ones(1,8),-0.5*ones(1,18)];
        bounds.parameter.upper = [2*ones(1,8),0.5*ones(1,18)];
    elseif ~handLoadControls
        bounds.parameter.lower = [-2*ones(1,8),-0.5*ones(1,6)];
        bounds.parameter.upper = [2*ones(1,8),0.5*ones(1,6)];
    %     bounds.parameter.lower = [-2*ones(1,8)];
    %     bounds.parameter.upper = [2*ones(1,8)];
    else
        bounds.parameter.lower = -2*ones(1,8);
        bounds.parameter.upper = 2*ones(1,8);
    end
    
end


%-------------------------------------------------------------------------%
%---------------------- Provide Guess of Solution ------------------------%
%-------------------------------------------------------------------------%
% solution.phase.control(:,30:99) = solution.phase.control(:,30:99)/10;
% Set initial guess either from previous optimal control solution or from
% the auxilary data which was calculated from a different model
iphase = 1;
if 1
    guess = solution;
else
    guess.phase(iphase).time    = ([auxdata.time]-(auxdata.tMax+auxdata.tMin)/2)./(auxdata.tMax-auxdata.tMin);
    guess.phase(iphase).state   = ([qGuess qpGuess qppGuess]-onescol*(auxdata.stateMax+auxdata.stateMin)/2)./(onescol*(auxdata.stateMax-auxdata.stateMin));% auxdata.Loads];% auxdata.Loadsp]
    if extraControls
        guess.phase(iphase).control = ([qpppGuess,torquesGuess]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
    elseif handLoadControls
        guess.phase(iphase).control = ([qpppGuess,onescol*zeros(1,12)]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
    else
        guess.phase(iphase).control = ([qpppGuess]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
    end
    
    guess.phase(iphase).integral = (10 - (auxdata.integralMax+auxdata.integralMin)/2)./(auxdata.integralMax - auxdata.integralMin);
    
    if calibrate % Must be active for model calibration
        if springPosParams
            guess.parameter = ([3e3 zeros(1,5) 0 0 1*ones(1,6) zeros(1,12)]-(auxdata.paramMax+auxdata.paramMin)/2)./(auxdata.paramMax-auxdata.paramMin);
        elseif ~handLoadControls
            guess.parameter = ([3e3 zeros(1,5) 0 0 1*ones(1,6)]-(auxdata.paramMax+auxdata.paramMin)/2)./(auxdata.paramMax-auxdata.paramMin);
%         guess.parameter = ([3e3 zeros(1,5) 0 0]-(auxdata.paramMax+auxdata.paramMin)/2)./(auxdata.paramMax-auxdata.paramMin);
        else
            guess.parameter = ([3e3 zeros(1,5) 0 0 ]-(auxdata.paramMax+auxdata.paramMin)/2)./(auxdata.paramMax-auxdata.paramMin);
        end
        
    end
    
end


%-------------------------------------------------------------------------%
%------------- Assemble Information into Problem Structure ---------------%
%-------------------------------------------------------------------------%
setup.name = 'openSim Full Body Motion';
setup.functions.continuous = @osimOptimalTrackingContinuous;
auxdata.ContinuousFunc = setup.functions.continuous;
setup.functions.endpoint = @osimOptimalTrackingEndpoint;
setup.auxdata = auxdata;
setup.bounds = bounds;
setup.guess = guess;
setup.nlp.solver = 'snopt';
% setup.nlp.ipoptoptions.linear_solver = 'ma57';
setup.nlp.ipoptoptions.tolerance = 1e-2;
setup.nlp.snoptoptions.tolerance = 5e-5;
setup.nlp.ipoptoptions.maxiterations = 200;
% setup.derivatives.stepsize1 = 1e-9;
setup.derivatives.supplier = 'sparseFD';
setup.derivatives.derivativelevel = 'first';
setup.derivatives.dependencies = 'sparse';
% setup.scales.method='automatic-bounds';
mesh.method = 'hp-PattersonRao';
mesh.tolerance       = 1e-4;
mesh.maxiterations    = 0;
mesh.colpointsmin    = 2;
mesh.colpointsmax    = 10; 
N                    = 5; %5
mesh.phase.colpoints = 10*ones(1,N); %10
mesh.phase.fraction  = ones(1,N)/N;
setup.method = 'RPM-integration';
setup.mesh = mesh;

%-------------------------------------------------------------------------%
%------------------------- Solve Problem Using GPOP2 ---------------------%
%-------------------------------------------------------------------------%
tic
output = gpops2(setup);
toc
solution = output.result.solution;
if calibrate
    save solution_calibrateSprings_snopt_v10HandControlsPt15.mat solution
    save output_calibrateSprings_snopt_v10HandControlsPt15.mat output
else
    save solution_osimOptTracking_verify_snopt_v0pt3.mat solution
    save output_osimOptTracking_verify_snopt_v0pt3.mat output
end


end

%-------------------------------------------------------------------------%
%------------------------------- Plot Solution ---------------------------%
%-------------------------------------------------------------------------%

%load output_osimOptTracking_calibrateSprings4.mat

tScale = 1;%auxdata.tScale;

input = solution;
if calibrate
    input.phase.parameter = solution.parameter;
end
input.auxdata = auxdata;

% outputContinuous = FullBodyOptimalControlContinuous_experimentalGRFs_Markers(input);
outputContinuous = osimOptimalTrackingContinuous(input);

onescol = ones(size(solution.phase.state(:,1)));

solution.phase.time = (solution.phase.time)*(auxdata.tMax-auxdata.tMin)+(auxdata.tMax+auxdata.tMin)/2;
solution.phase.state = (solution.phase.state).*(onescol*(auxdata.stateMax-auxdata.stateMin))+onescol*(auxdata.stateMax+auxdata.stateMin)/2;
solution.phase.control = (solution.phase.control).*(onescol*(auxdata.controlMax-auxdata.controlMin))+onescol*(auxdata.controlMax+auxdata.controlMin)/2;
if calibrate
    solution.parameter = (solution.parameter(1,:)).*((input.auxdata.paramMax-input.auxdata.paramMin))+(input.auxdata.paramMax+input.auxdata.paramMin)/2;
end

mkrDataSplined = interp1(auxdata.time,auxdata.mkrData,solution.phase.time,'pchip');

mkrDataSplined = fnval(auxdata.spMkrData,solution.phase.time)';

error = mkrDataSplined-outputContinuous.markers;

% Calculate RMS error for each marker at each time frame
for i = 1:size(mkrDataSplined,2)/3
    RMSError(:,i) = sum(error(:,(i-1)*3+1:3*i).^2,2).^(1/2);
end

GRFsSplit = outputContinuous.GRFsSplit;
figure % 1
plot(solution.phase.time,GRFsSplit(:,1:3)+GRFsSplit(:,4:6),'r')
hold on
plot(auxdata.time,auxdata.grfDataR(:,1:3),'b')

figure % 2
plot(solution.phase.time,GRFsSplit(:,13:15)+GRFsSplit(:,16:18),'r')
hold on
plot(auxdata.time,auxdata.grfDataL(:,1:3),'b')

figure % 3
plot(solution.phase.time,GRFsSplit(:,7:9)+GRFsSplit(:,10:12),'r')
hold on
plot(auxdata.time,auxdata.grfDataR(:,4:6),'b')

figure % 4
plot(solution.phase.time,GRFsSplit(:,19:21)+GRFsSplit(:,22:24),'r')
hold on
plot(auxdata.time,auxdata.grfDataL(:,4:6),'b')

% figure % 5
% plot(solution.phase.time, solution.phase.control(:,1:23),'r')
% hold on
% plot(auxdata.time, auxdata.qppp,'b')

figure % 6
for i = 1:31
    
    subplot(6,6,i)
    plot(solution.phase.time/solution.phase.time(end), solution.phase.state(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.q(:,i),'b')
    plot(auxdata.time/auxdata.time(end), qGuess(:,i),'b')
    title(auxdata.qLabels(i));
end

figure % 7
for i = 1:31
    
    subplot(6,6,i)
    plot(solution.phase.time/solution.phase.time(end), solution.phase.control(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.qpp(:,i),'b')
    plot(auxdata.time/auxdata.time(end), qpppGuess(:,i),'b')
    title(auxdata.qLabels(i));
end

figure % 8
for i = 1:23
    subplot(5,5,i)
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.torques(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.idData(:,i),'b')
    plot(auxdata.time/auxdata.time(end), auxdata.torques(:,i),'b')
    title(auxdata.torqueNames(i));
end

figure % 9
plot3(auxdata.springsPoly_r(:,1),auxdata.springsPoly_r(:,3),outputContinuous.kVal(1:(auxdata.numSpringsBody(1)+auxdata.numSpringsBody(2))),'x')
figure % 10
plot3(auxdata.springsPoly_l(:,1),auxdata.springsPoly_l(:,3),outputContinuous.kVal((1+auxdata.numSpringsBody(1)+auxdata.numSpringsBody(2)):end),'rx')

figure % 11
for iii = 1:3
    subplot(2,3,iii);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsR(:,iii));
    subplot(2,3,iii+3);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsL(:,iii));
end

figure % 12
for iii = 1:3
    subplot(2,3,iii);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsR(:,iii+3));
    subplot(2,3,iii+3);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsL(:,iii+3));
end

ToeTorques = outputContinuous.torques(:,[7 14]);

ToeKinematics = solution.phase.state(:,[13 20 42 49]);

% Mat1 = [ones(501,1) ToeKinematics(:,[1 3])];
% Mat2 = [ones(501,1) ToeKinematics(:,[2 4])];
% 
% ApproxTorques(:,1) = Mat1*(Mat1\ToeTorques(:,1));
% ApproxTorques(:,2) = Mat2*(Mat2\ToeTorques(:,2));
% 
% [spIDnew] = spaps(solution.phase.time,outputContinuous.Torques',.5);
% 
% control = fnval(spIDnew,auxdata.time)';
% 
% NewID = control(:,[8 9 11 12 13 1 2 4 5 6]);

if calibrate
    % Get spring constants from continuous function
    kVal = outputContinuous.kVal;
    yVal = outputContinuous.yVal;
    if ~handLoadControls
        handSpring_k = outputContinuous.handSpring_k;
        handSpring_tk = outputContinuous.handSpring_tk;
    end
    
    % Output spring constants
    if handLoadControls
        save SpringConstants.mat kVal yVal
    else
        save SpringConstants.mat kVal yVal handSpring_k handSpring_tk
    end
    

    % save control.mat control

    % save NewIDLoads.mat NewID

    outputIKMot('IK.mot',solution.phase.state(:,1:31),solution.phase.time)
    
end

%% Calculate Error in Ground Reaction Calibration

stanceIndices_r = 2:31;
stanceIndices_l = [19:50,1:6];

% Simulation GRFs
grfStance_r = GRFsSplit(stanceIndices_r,1:3)+GRFsSplit(stanceIndices_r,4:6);
grfStance_l = GRFsSplit(stanceIndices_l,13:15)+GRFsSplit(stanceIndices_l,16:18);

% Simulation GRMs
grmStance_r = GRFsSplit(stanceIndices_r,7:9)+GRFsSplit(stanceIndices_r,10:12);
grmStance_l = GRFsSplit(stanceIndices_l,19:21)+GRFsSplit(stanceIndices_l,22:24);

% Measured GRFs
measuredGrf_r = fnval(auxdata.spGRFR,solution.phase.time)';
measuredGrf_l = fnval(auxdata.spGRFL,solution.phase.time)';
measuredGrfStance_r = measuredGrf_r(stanceIndices_r,1:3);
measuredGrfStance_l = measuredGrf_l(stanceIndices_l,1:3);

% Measured GRMs
measuredGrmStance_r = measuredGrf_r(stanceIndices_r,4:6);
measuredGrmStance_l = measuredGrf_l(stanceIndices_l,4:6);

% Error in GRFs and GRMs
grfStanceError_r = grfStance_r - measuredGrfStance_r(:,1:3);
grfStanceError_l = grfStance_l - measuredGrfStance_l(:,1:3);
grmStanceError_r = grmStance_r - measuredGrmStance_r(:,1:3);
grmStanceError_l = grmStance_l - measuredGrmStance_l(:,1:3);

% Calculate COP and free torque error
% qx = px + (Tz-Fx*(py-qy))/Fy
% qz = pz - (Tx+Fz*(py-qy))/Fy
% TyFree = Ty + Fx*(pz-qz) - Fz*(px-qx)
nonZeroGrfy_r = grfStance_r(:,2);
nonZeroGrfy_r(nonZeroGrfy_r < 1) = 1;
nonZeroGrfy_l = grfStance_l(:,2);
nonZeroGrfy_l(nonZeroGrfy_l < 1) = 1;
nonZeroMeasuredGrfy_r = measuredGrfStance_r(:,2);
nonZeroMeasuredGrfy_r(nonZeroMeasuredGrfy_r < 1) = 1;
nonZeroMeasuredGrfy_l = measuredGrfStance_l(:,2);
nonZeroMeasuredGrfy_l(nonZeroMeasuredGrfy_l < 1) = 1;

% Get Stance MCs
mc_r = fnval(auxdata.spMcr,solution.phase.time)';
mc_l = fnval(auxdata.spMcl,solution.phase.time)';
mcStance_r = mc_r(stanceIndices_r,:);
mcStance_l = mc_l(stanceIndices_l,:);

% Calculate Relative COP
copx_r = mcStance_r(:,1) + (grmStance_r(:,3)./nonZeroGrfy_r);
copx_l = mcStance_l(:,1) + (grmStance_l(:,3)./nonZeroGrfy_l);
copz_r = mcStance_r(:,3) - (grmStance_r(:,1)./nonZeroGrfy_r);
copz_l = mcStance_l(:,3) - (grmStance_l(:,1)./nonZeroGrfy_l);
measuredCopx_r = mcStance_r(:,1) + (measuredGrmStance_r(:,3)./nonZeroMeasuredGrfy_r);
measuredCopx_l = mcStance_l(:,1) + (measuredGrmStance_l(:,3)./nonZeroMeasuredGrfy_l);
measuredCopz_r = mcStance_r(:,3) - (measuredGrmStance_r(:,1)./nonZeroMeasuredGrfy_r);
measuredCopz_l = mcStance_l(:,3) - (measuredGrmStance_l(:,1)./nonZeroMeasuredGrfy_l);

% COP1xCalc = (h*loadsForECCalc(:,1) + loadsForECCalc(:,6))./loadsForECCalc(:,2) + ECs(1); % px + (h*Fx + Tz)./Fy
% COP1zCalc = (-h*loadsForECCalc(:,3) - loadsForECCalc(:,4))./loadsForECCalc(:,2) + ECs(3); % pz - (h*Fz + Tx)./Fy
% 
% COP2xCalc = (h*loadsForECCalc(:,7) + loadsForECCalc(:,12))./loadsForECCalc(:,8) + ECs(4);
% COP2zCalc = (-h*loadsForECCalc(:,9) - loadsForECCalc(:,10))./loadsForECCalc(:,8) + ECs(6);


% Calculate COP Error
copxError_r = copx_r - measuredCopx_r;
copxError_l = copx_l - measuredCopx_l;
copzError_r = copz_r - measuredCopz_r;
copzError_l = copz_l - measuredCopz_l;

copError_r = [copxError_r,copzError_r];
copError_l = [copxError_l,copzError_l];

% Calculate Free Torque
freeTorque_r = grmStance_r(:,2) + grfStance_r(:,1).*(mcStance_r(:,3)-copz_r) - grfStance_r(:,3).*(mcStance_r(:,1)-copx_r);
freeTorque_l = grmStance_l(:,2) + grfStance_l(:,1).*(mcStance_l(:,3)-copz_l) - grfStance_l(:,3).*(mcStance_l(:,1)-copx_l);
measuredFreeTorque_r = measuredGrmStance_r(:,2) + measuredGrfStance_r(:,1).*(mcStance_r(:,3)-measuredCopz_r) - measuredGrfStance_r(:,3).*(mcStance_r(:,1)-measuredCopx_r);
measuredFreeTorque_l = measuredGrmStance_l(:,2) + measuredGrfStance_l(:,1).*(mcStance_l(:,3)-measuredCopz_l) - measuredGrfStance_l(:,3).*(mcStance_l(:,1)-measuredCopx_l);

% Calculate Free Torque Error
freeTorqueError_r = freeTorque_r - measuredFreeTorque_r;
freeTorqueError_l = freeTorque_l - measuredFreeTorque_l;

% Calculate RMS Errors
grfStanceRMSE_r = sqrt(mean(grfStanceError_r.^2,1));
grfStanceRMSE_l = sqrt(mean(grfStanceError_l.^2,1));
grmStanceRMSE_r = sqrt(mean(grmStanceError_r.^2,1));
grmStanceRMSE_l = sqrt(mean(grmStanceError_l.^2,1));
copRMSE_r = sqrt(mean(copError_r.^2,1));
copRMSE_l = sqrt(mean(copError_l.^2,1));
freeTorqueRMSE_r = sqrt(mean(freeTorqueError_r.^2,1));
freeTorqueRMSE_l = sqrt(mean(freeTorqueError_l.^2,1));


% Print Errors for Comparison
apForceRMSE_r = grfStanceRMSE_r(1)
normForceRMSE_r = grfStanceRMSE_r(2)
mlForceRMSE_r =  grfStanceRMSE_r(3)
xMomentRMSE_r = grmStanceRMSE_r(1)
yMomentRMSE_r = grmStanceRMSE_r(2)
zMomentRMSE_r = grmStanceRMSE_r(3)
% apCopRMSE_r = copRMSE_r(1)*1e3
% mlCopRMSE_r = copRMSE_r(2)*1e3
% freeTorqueRMSE_r


apForceRMSE_l = grfStanceRMSE_l(1)
normForceRMSE_l = grfStanceRMSE_l(2)
mlForceRMSE_l =  grfStanceRMSE_l(3)
xMomentRMSE_l = grmStanceRMSE_l(1)
yMomentRMSE_l = grmStanceRMSE_l(2)
zMomentRMSE_l = grmStanceRMSE_l(3)
% apCopRMSE_l = copRMSE_l(1)*1e3
% mlCopRMSE_l = copRMSE_l(2)*1e3
% freeTorqueRMSE_l

figure;
plot(copx_r);
hold on
plot(measuredCopx_r,'k');
figure;
plot(copx_l);
hold on;
plot(measuredCopx_l,'k');
figure;
plot(copz_r);
hold on
plot(measuredCopz_r,'k');
figure;
plot(copz_l);
hold on;
plot(measuredCopz_l,'k');


